from pathlib import Path
from pydantic import BaseModel
from typing import Dict
from rich.console import Console
from dataclasses import dataclass
import subprocess

@dataclass
class Task:
    name: str
    module: str
    parameters_and_flags: Dict[str, str | bool]
    use_underscores: bool = False

    def __post_init__(self):
        self._parameters = {}
        self._flags = []
        self._get_flags_and_params()

    def _get_flags_and_params(self):
        for k, v in self.parameters_and_flags.items():
            if isinstance(v, bool):
                if v:
                    self._flags.append(k)
            else:
                self._parameters[k] = v

    @property
    def parameters(self):
        list_of_tuples = [
            (f"--{k.replace('_', '-')}", f"{v}") if not self.use_underscores else (f"--{k}", f"{v}")
            for k, v in self._parameters.items()
        ]
        # flatten: 
        return [j
                for i in list_of_tuples
                for j in i]

    @property
    def flags(self):
        return [
            f"--{flag.replace('_', '-')}" if not self.use_underscores else f"--{flag}"
            for flag in self._flags
        ]

    def run(self, path_to_python: Path = Path("python")):
        console = Console()
        console.print(f"[green bold]Running {self.name}...[/green bold]")
        proc = subprocess.run([str(path_to_python), "-m", self.module, *self.parameters, *self.flags], check=True)
        return proc.returncode

    def __rshift__(self, other):
        return TaskChain(self, other)


@dataclass
class TaskChain:
    left: Task
    right: Task

    def run(self, path_to_python: Path = Path("python")):
        self.left.run(path_to_python)
        self.right.run(path_to_python)

    def __rshift__(self, other):
        return TaskChain(self, other)



from rich.console import Console


REGISTERED_WORKFLOWS = {}


def workflow(func):
    console = Console()
    def wrapper(*args, **kwargs):
        console.print(f"[blue]Running workflow {func.__name__}[/blue]")
        func(*args, **kwargs)
    REGISTERED_WORKFLOWS[func.__name__] = wrapper
    return wrapper


def get_workflow(name):
    return REGISTERED_WORKFLOWS[name]



from pathlib import Path
import click
import importlib.util
import sys
from rich.console import Console
from sewerpipe.workflows import get_workflow
from sewerpipe.task import Task


def magical_module_import(path: Path):
    module_name = "sewerpipe.current_workflow"
    spec = importlib.util.spec_from_file_location(module_name, path)
    module = importlib.util.module_from_spec(spec)
    sys.modules[module_name] = module
    spec.loader.exec_module(module)
    return module


@click.command()
@click.option("-p", "--path", type=click.Path(exists=True),
              help="Path to the workflow definition file.")
@click.option("-i", "--python-interpreter", type=click.Path(),
              help="Path to the python interpreter.",
              default="python")
@click.option("-w", "--workflow", type=str,
              help="Workflow name.", default="workflow")
def run(path: Path, python_interpreter: Path, workflow: str):
    console = Console()
    console.print(f"[blue]Provided path: {path}[/blue]")

    # This blackmagic f***ery is needed to import the module from a given path:
    module = magical_module_import(path)

    # Here's where a registering decorator comes in handy:
    workflow = get_workflow(workflow)

    # Workflows can optionally return a value:
    res = workflow()

    # Print the result:
    console.print(f"[green]Workflow {workflow} has returned {res}[/green]")


@click.command()
@click.option("-p", "--path", type=click.Path(exists=True),
              help="Path to the workflow definition file.")
@click.option("-o", "--output", type=click.Path(),
                help="Path to the output file.")
@click.option("--to", type=click.Choice(["vscode", "airflow"]),
              help="Output format.")
@click.option("--print-only", is_flag=True, help="Print the output instead of writing to a file.")
@click.option("--launch-json-version", default="0.2.0", help="Version of the launch.json file.")
def convert(path: Path, output: Path, to: str, print_only: bool, launch_json_version: str):
    console = Console()
    match to:
        case "vscode":
            from sewerpipe.vscode import generate_launch_json
            module = magical_module_import(path)
            all_tasks = dict([(name, klass) for name, klass in module.__dict__.items() if isinstance(klass, Task)])
            console.print(f"[green]Found tasks: {[f'{name} ({task.name})' for name, task in all_tasks.items()]}[/green]")
            if len(all_tasks) == 0:
                console.print("[red]No tasks found in the provided module.[/red]")
                return
            ret = generate_launch_json(all_tasks.values(),
                                       output,
                                       launch_json_version,
                                       "write" if not print_only else "return")
            if print_only:
                console.print(ret)
        case "airflow":
            # from sewerpipe.airflow import create_airflow_tasks
            raise NotImplementedError("Airflow conversion is supported only in library usage.")
        case _:
            raise ValueError(f"Invalid value for 'to': {to}")


@click.group()
def main():
    pass


main.add_command(run)
main.add_command(convert)


if __name__ == "__main__":
    main()



from typing import List, Dict, Literal
from pathlib import Path
import json
from rich.console import Console
from sewerpipe.task import Task


def generate_launch_json(tasks: List[Task] | Task,
                         launch_json_path: Path = ".vscode/launch.json",
                         launch_json_version: str = "0.2.0",
                         return_or_write: Literal["return", "write"] = "write"):
    console = Console()
    if isinstance(tasks, Task):
        tasks = [tasks]
    launch_config = {
        "version": launch_json_version,
        "configurations": [
            {
                "name": f"{wfcfg.name}",
                "type": "python",
                "request": "launch",
                "module": f"{wfcfg.module}",
                "args": wfcfg.parameters + wfcfg.flags,
                "console": "integratedTerminal"
            }
        for wfcfg in tasks]
    }

    match return_or_write:
        case "write":
            console.print(f"[blue]Writing launch.json to {launch_json_path}[/blue]")
            with open(launch_json_path, 'w') as file:
                json.dump(launch_config, file, indent=4)
        case "return":
            return launch_config
        case _:
            raise ValueError(f"Invalid value for return_or_write: {return_or_write}")


if __name__ == "__main__":
    generate_launch_json('workflow_config.json', '.vscode/launch.json')



from pydantic import BaseModel
from confuk.parse import parse_config
from sewerpipe.task import Task
from typing import List


class Config(BaseModel):
    """Config for all workflows."""
    tasks: List[Task]

    @classmethod
    def from_file(cls, file_path: str):
        return parse_config(file_path, cls)



from pathlib import Path
from datetime import datetime
from airflow.decorators import task as airflow_task
from sewerpipe.task import Task


def create_airflow_task(task: Task,
                        path_to_python: Path = Path("python")):
    if str(path_to_python) == "python":
        @airflow_task(task_id=task.name)
        def _task():
            task.run()
    else:
        @airflow_task.external_python(task_id=task.name, python_path=path_to_python)
        def _task():
            task.run(path_to_python=path_to_python)
    return _task


def create_airflow_tasks(tasks: list[Task],
                         path_to_python: Path = Path("python")):
    return (create_airflow_task(task, path_to_python) for task in tasks)



import click
from rich.console import Console


@click.command()
@click.option("--verbose", is_flag=True, help="Will print verbose messages.")
@click.option("--name", "-n", help="Who are you?")
def main(verbose, name):
    console = Console()
    args = dict(verbose=verbose, name=name)
    console.print(f"This script has been called with the following arguments: {args}")


if __name__ == "__main__":
    main()
